@@ -2,7 +2,7 @@ use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex};
 use zellij_tile::data::Palette;
 
-use zellij_server::panes::TerminalPane;
+use zellij_server::panes::{LinkHandler, TerminalPane};
 use zellij_utils::pane_size::{Dimension, PaneGeom, Size};
 use zellij_utils::{vte, zellij_tile};
 
@@ -12,6 +12,9 @@ use std::net::TcpStream;
 
 use std::path::Path;
 
+use std::cell::RefCell;
+use std::rc::Rc;
+
 const ZELLIJ_EXECUTABLE_LOCATION: &str = "/usr/src/zellij/x86_64-unknown-linux-musl/release/zellij";
 const ZELLIJ_LAYOUT_PATH: &str = "/usr/src/zellij/fixtures/layouts";
 const CONNECTION_STRING: &str = "127.0.0.1:2222";
@@ -141,16 +144,23 @@ fn read_from_channel(
     let thread = std::thread::Builder::new()
         .name("read_thread".into())
         .spawn({
+            let pane_geom = *pane_geom;
             let should_keep_running = should_keep_running.clone();
             let channel = channel.clone();
             let last_snapshot = last_snapshot.clone();
             let cursor_coordinates = cursor_coordinates.clone();
-            let mut vte_parser = vte::Parser::new();
-            let mut terminal_output =
-                TerminalPane::new(0, *pane_geom, Palette::default(), 0, String::new()); // 0 is the pane index
-            let mut retries_left = 3;
             move || {
+                let mut retries_left = 3;
                 let mut should_sleep = false;
+                let mut vte_parser = vte::Parser::new();
+                let mut terminal_output = TerminalPane::new(
+                    0,
+                    pane_geom,
+                    Palette::default(),
+                    0,
+                    String::new(),
+                    Rc::new(RefCell::new(LinkHandler::new())),
+                ); // 0 is the pane index
                 loop {
                     if !should_keep_running.load(Ordering::SeqCst) {
                         break;
