@@ -1,3 +1,5 @@
+use std::cell::RefCell;
+use std::rc::Rc;
 use unicode_width::UnicodeWidthChar;
 
 use std::{
@@ -20,6 +22,7 @@ use vte::{Params, Perform};
 use zellij_tile::data::{Palette, PaletteColor};
 use zellij_utils::{consts::VERSION, shared::version_number};
 
+use crate::output::{CharacterChunk, OutputBuffer};
 use crate::panes::alacritty_functions::{parse_number, xparse_color};
 use crate::panes::link_handler::LinkHandler;
 use crate::panes::selection::Selection;
@@ -267,114 +270,13 @@ fn subtract_isize_from_usize(u: usize, i: isize) -> usize {
     }
 }
 
-#[derive(Debug)]
-pub struct CharacterChunk {
-    pub terminal_characters: Vec<TerminalCharacter>,
-    pub x: usize,
-    pub y: usize,
-}
-
-#[derive(Clone, Debug)]
-pub struct OutputBuffer {
-    changed_lines: Vec<usize>, // line index
-    should_update_all_lines: bool,
-}
-
-impl Default for OutputBuffer {
-    fn default() -> Self {
-        OutputBuffer {
-            changed_lines: vec![],
-            should_update_all_lines: true, // first time we should do a full render
-        }
-    }
-}
-
-impl OutputBuffer {
-    pub fn update_line(&mut self, line_index: usize) {
-        if !self.should_update_all_lines {
-            self.changed_lines.push(line_index);
-        }
-    }
-    pub fn update_all_lines(&mut self) {
-        self.clear();
-        self.should_update_all_lines = true;
-    }
-    pub fn clear(&mut self) {
-        self.changed_lines.clear();
-        self.should_update_all_lines = false;
-    }
-    pub fn changed_chunks_in_viewport(
-        &self,
-        viewport: &[Row],
-        viewport_width: usize,
-        viewport_height: usize,
-    ) -> Vec<CharacterChunk> {
-        if self.should_update_all_lines {
-            let mut changed_chunks = Vec::with_capacity(viewport.len());
-            for line_index in 0..viewport_height {
-                let terminal_characters =
-                    self.extract_line_from_viewport(line_index, viewport, viewport_width);
-                changed_chunks.push(CharacterChunk {
-                    x: 0,
-                    y: line_index,
-                    terminal_characters,
-                });
-            }
-            changed_chunks
-        } else {
-            let mut line_changes = self.changed_lines.to_vec();
-            line_changes.sort_unstable();
-            line_changes.dedup();
-            let mut changed_chunks = Vec::with_capacity(line_changes.len());
-            for line_index in line_changes {
-                let terminal_characters =
-                    self.extract_line_from_viewport(line_index, viewport, viewport_width);
-                changed_chunks.push(CharacterChunk {
-                    x: 0,
-                    y: line_index,
-                    terminal_characters,
-                });
-            }
-            changed_chunks
-        }
-    }
-    fn extract_characters_from_row(
-        &self,
-        row: &Row,
-        viewport_width: usize,
-    ) -> Vec<TerminalCharacter> {
-        let mut terminal_characters: Vec<TerminalCharacter> = row.columns.iter().copied().collect();
-        // pad row
-        let row_width = row.width();
-        if row_width < viewport_width {
-            let mut padding = vec![EMPTY_TERMINAL_CHARACTER; viewport_width - row_width];
-            terminal_characters.append(&mut padding);
-        }
-        terminal_characters
-    }
-    fn extract_line_from_viewport(
-        &self,
-        line_index: usize,
-        viewport: &[Row],
-        viewport_width: usize,
-    ) -> Vec<TerminalCharacter> {
-        match viewport.get(line_index) {
-            // TODO: iterator?
-            Some(row) => self.extract_characters_from_row(row, viewport_width),
-            None => {
-                vec![EMPTY_TERMINAL_CHARACTER; viewport_width]
-            }
-        }
-    }
-}
-
 #[derive(Clone)]
 pub struct Grid {
     lines_above: VecDeque<Row>,
     viewport: Vec<Row>,
     lines_below: Vec<Row>,
     horizontal_tabstops: BTreeSet<usize>,
-    alternative_lines_above_viewport_and_cursor: Option<(VecDeque<Row>, Vec<Row>, Cursor)>,
+    alternate_lines_above_viewport_and_cursor: Option<(VecDeque<Row>, Vec<Row>, Cursor)>,
     cursor: Cursor,
     saved_cursor_position: Option<Cursor>,
     // FIXME: change scroll_region to be (usize, usize) - where the top line is always the first
@@ -401,7 +303,7 @@ pub struct Grid {
     pub selection: Selection,
     pub title: Option<String>,
     pub is_scrolled: bool,
-    pub link_handler: LinkHandler,
+    pub link_handler: Rc<RefCell<LinkHandler>>,
     pub ring_bell: bool,
     scrollback_buffer_lines: usize,
 }
@@ -420,7 +322,12 @@ impl Debug for Grid {
 }
 
 impl Grid {
-    pub fn new(rows: usize, columns: usize, colors: Palette) -> Self {
+    pub fn new(
+        rows: usize,
+        columns: usize,
+        colors: Palette,
+        link_handler: Rc<RefCell<LinkHandler>>,
+    ) -> Self {
         Grid {
             lines_above: VecDeque::with_capacity(
                 // .get_or_init() is used instead of .get().unwrap() to prevent
@@ -442,7 +349,7 @@ impl Grid {
             erasure_mode: false,
             insert_mode: false,
             disable_linewrap: false,
-            alternative_lines_above_viewport_and_cursor: None,
+            alternate_lines_above_viewport_and_cursor: None,
             clear_viewport_before_rendering: false,
             active_charset: Default::default(),
             pending_messages_to_pty: vec![],
@@ -453,7 +360,7 @@ impl Grid {
             title: None,
             changed_colors: None,
             is_scrolled: false,
-            link_handler: Default::default(),
+            link_handler,
             ring_bell: false,
             scrollback_buffer_lines: 0,
         }
@@ -673,7 +580,7 @@ impl Grid {
             return;
         }
         self.selection.reset();
-        if new_columns != self.width {
+        if new_columns != self.width && self.alternate_lines_above_viewport_and_cursor.is_none() {
             self.horizontal_tabstops = create_horizontal_tabstops(new_columns);
             let mut cursor_canonical_line_index = self.cursor_canonical_line_index();
             let cursor_index_in_canonical_line = self.cursor_index_in_canonical_line();
@@ -717,7 +624,7 @@ impl Grid {
                     }
                 }
                 if let Some(trim_at) = trim_at {
-                    line.columns.truncate(trim_at);
+                    line.truncate(trim_at);
                 }
             }
 
@@ -786,6 +693,16 @@ impl Grid {
             }
             self.cursor.y = new_cursor_y;
             self.cursor.x = new_cursor_x;
+        } else if new_columns != self.width
+            && self.alternate_lines_above_viewport_and_cursor.is_some()
+        {
+            // in alternate screen just truncate exceeding width
+            for row in &mut self.viewport {
+                if row.width() >= new_columns {
+                    let truncate_at = row.position_accounting_for_widechars(new_columns);
+                    row.columns.truncate(truncate_at);
+                }
+            }
         }
         if new_rows != self.height {
             let current_viewport_row_count = self.viewport.len();
@@ -808,12 +725,17 @@ impl Grid {
                     } else {
                         self.cursor.y -= row_count_to_transfer;
                     }
-                    transfer_rows_from_viewport_to_lines_above(
-                        &mut self.viewport,
-                        &mut self.lines_above,
-                        row_count_to_transfer,
-                        new_columns,
-                    );
+                    if self.alternate_lines_above_viewport_and_cursor.is_none() {
+                        transfer_rows_from_viewport_to_lines_above(
+                            &mut self.viewport,
+                            &mut self.lines_above,
+                            row_count_to_transfer,
+                            new_columns,
+                        );
+                    } else {
+                        // in alternate screen, no scroll buffer, so just remove lines
+                        self.viewport.drain(0..row_count_to_transfer);
+                    }
                 }
                 Ordering::Equal => {}
             }
@@ -850,10 +772,14 @@ impl Grid {
         }
         lines
     }
-    pub fn read_changes(&mut self) -> Vec<CharacterChunk> {
-        let changes =
-            self.output_buffer
-                .changed_chunks_in_viewport(&self.viewport, self.width, self.height);
+    pub fn read_changes(&mut self, x_offset: usize, y_offset: usize) -> Vec<CharacterChunk> {
+        let changes = self.output_buffer.changed_chunks_in_viewport(
+            &self.viewport,
+            self.width,
+            self.height,
+            x_offset,
+            y_offset,
+        );
         self.output_buffer.clear();
         changes
     }
@@ -916,16 +842,11 @@ impl Grid {
         }
     }
     pub fn fill_viewport(&mut self, character: TerminalCharacter) {
-        let row_count_to_transfer = self.viewport.len();
-        let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
-            &mut self.viewport,
-            &mut self.lines_above,
-            row_count_to_transfer,
-            self.width,
-        );
-
-        self.scrollback_buffer_lines =
-            subtract_isize_from_usize(self.scrollback_buffer_lines, transferred_rows_count);
+        if self.alternate_lines_above_viewport_and_cursor.is_some() {
+            self.viewport.clear();
+        } else {
+            self.transfer_rows_to_lines_above(self.viewport.len())
+        };
 
         for _ in 0..self.height {
             let columns = VecDeque::from(vec![character; self.width]);
@@ -947,17 +868,12 @@ impl Grid {
                     return;
                 }
                 if scroll_region_bottom == self.height - 1 && scroll_region_top == 0 {
-                    let row_count_to_transfer = 1;
-                    let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
-                        &mut self.viewport,
-                        &mut self.lines_above,
-                        row_count_to_transfer,
-                        self.width,
-                    );
-                    self.scrollback_buffer_lines = subtract_isize_from_usize(
-                        self.scrollback_buffer_lines,
-                        transferred_rows_count,
-                    );
+                    if self.alternate_lines_above_viewport_and_cursor.is_none() {
+                        self.transfer_rows_to_lines_above(1);
+                    } else {
+                        self.viewport.remove(0);
+                    }
+
                     let columns = VecDeque::from(vec![EMPTY_TERMINAL_CHARACTER; self.width]);
                     self.viewport.push(Row::from_columns(columns).canonical());
                     self.selection.move_up(1);
@@ -984,15 +900,11 @@ impl Grid {
         }
         if self.cursor.y == self.height - 1 {
             if self.scroll_region.is_none() {
-                let row_count_to_transfer = 1;
-                let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
-                    &mut self.viewport,
-                    &mut self.lines_above,
-                    row_count_to_transfer,
-                    self.width,
-                );
-                self.scrollback_buffer_lines =
-                    subtract_isize_from_usize(self.scrollback_buffer_lines, transferred_rows_count);
+                if self.alternate_lines_above_viewport_and_cursor.is_none() {
+                    self.transfer_rows_to_lines_above(1);
+                } else {
+                    self.viewport.remove(0);
+                }
 
                 self.selection.move_up(1);
             }
@@ -1055,22 +967,18 @@ impl Grid {
     pub fn add_character(&mut self, terminal_character: TerminalCharacter) {
         // TODO: try to separate adding characters from moving the cursors in this function
         let character_width = terminal_character.width;
-        if self.cursor.x >= self.width {
+        if self.cursor.x + character_width > self.width {
             if self.disable_linewrap {
                 return;
             }
             // line wrap
             self.cursor.x = 0;
             if self.cursor.y == self.height - 1 {
-                let row_count_to_transfer = 1;
-                let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
-                    &mut self.viewport,
-                    &mut self.lines_above,
-                    row_count_to_transfer,
-                    self.width,
-                );
-                self.scrollback_buffer_lines =
-                    subtract_isize_from_usize(self.scrollback_buffer_lines, transferred_rows_count);
+                if self.alternate_lines_above_viewport_and_cursor.is_none() {
+                    self.transfer_rows_to_lines_above(1);
+                } else {
+                    self.viewport.remove(0);
+                }
                 let wrapped_row = Row::new(self.width);
                 self.viewport.push(wrapped_row);
                 self.selection.move_up(1);
@@ -1098,7 +1006,7 @@ impl Grid {
         self.viewport.get(y).unwrap().absolute_character_index(x)
     }
     pub fn move_cursor_forward_until_edge(&mut self, count: usize) {
-        let count_to_move = std::cmp::min(count, self.width - (self.cursor.x));
+        let count_to_move = std::cmp::min(count, self.width - self.cursor.x);
         self.cursor.x += count_to_move;
     }
     pub fn replace_characters_in_line_after_cursor(&mut self, replace_with: TerminalCharacter) {
@@ -1145,6 +1053,11 @@ impl Grid {
         }
         self.output_buffer.update_all_lines();
     }
+    fn clear_lines_above(&mut self) {
+        self.lines_above.clear();
+        self.scrollback_buffer_lines = self.recalculate_scrollback_buffer_count();
+    }
+
     fn pad_current_line_until(&mut self, position: usize) {
         let current_row = self.viewport.get_mut(self.cursor.y).unwrap();
         for _ in current_row.len()..position {
@@ -1360,7 +1273,7 @@ impl Grid {
         self.lines_above = VecDeque::with_capacity(*SCROLL_BUFFER_SIZE.get().unwrap());
         self.lines_below = vec![];
         self.viewport = vec![Row::new(self.width).canonical()];
-        self.alternative_lines_above_viewport_and_cursor = None;
+        self.alternate_lines_above_viewport_and_cursor = None;
         self.cursor_key_mode = false;
         self.scroll_region = None;
         self.clear_viewport_before_rendering = true;
@@ -1378,27 +1291,27 @@ impl Grid {
         self.preceding_char = Some(terminal_character);
     }
     pub fn start_selection(&mut self, start: &Position) {
-        let old_selection = self.selection.clone();
+        let old_selection = self.selection;
         self.selection.start(*start);
         self.update_selected_lines(&old_selection, &self.selection.clone());
         self.mark_for_rerender();
     }
     pub fn update_selection(&mut self, to: &Position) {
-        let old_selection = self.selection.clone();
+        let old_selection = self.selection;
         self.selection.to(*to);
         self.update_selected_lines(&old_selection, &self.selection.clone());
         self.mark_for_rerender();
     }
 
     pub fn end_selection(&mut self, end: Option<&Position>) {
-        let old_selection = self.selection.clone();
+        let old_selection = self.selection;
         self.selection.end(end);
         self.update_selected_lines(&old_selection, &self.selection.clone());
         self.mark_for_rerender();
     }
 
     pub fn reset_selection(&mut self) {
-        let old_selection = self.selection.clone();
+        let old_selection = self.selection;
         self.selection.reset();
         self.update_selected_lines(&old_selection, &self.selection.clone());
         self.mark_for_rerender();
@@ -1453,25 +1366,31 @@ impl Grid {
                 continue;
             };
 
-            let excess_width = row.excess_width();
-            let mut line: Vec<TerminalCharacter> = row.columns.iter().copied().collect();
-            // pad line
-            line.resize(
-                self.width.saturating_sub(excess_width),
-                EMPTY_TERMINAL_CHARACTER,
-            );
-
             let mut terminal_col = 0;
-            for terminal_character in line {
+            for terminal_character in &row.columns {
                 if (start_column..end_column).contains(&terminal_col) {
                     line_selection.push(terminal_character.character);
                 }
 
                 terminal_col += terminal_character.width;
             }
-            selection.push(String::from(line_selection.trim_end()));
+
+            if row.is_canonical {
+                selection.push(line_selection);
+            } else {
+                // rejoin wrapped lines if possible
+                match selection.last_mut() {
+                    Some(previous_line) => previous_line.push_str(&line_selection),
+                    None => selection.push(line_selection),
+                }
+            }
         }
 
+        // TODO: distinguish whitespace that was output explicitly vs implicitly (e.g add_newline)
+        // for example: echo "     " vs empty lines
+        // for now trim after building the selection to handle whitespace in wrapped lines
+        let selection: Vec<_> = selection.iter().map(|l| l.trim_end()).collect();
+
         Some(selection.join("\n"))
     }
 
@@ -1496,6 +1415,17 @@ impl Grid {
             self.title = Some(popped_title);
         }
     }
+    fn transfer_rows_to_lines_above(&mut self, count: usize) {
+        let transferred_rows_count = transfer_rows_from_viewport_to_lines_above(
+            &mut self.viewport,
+            &mut self.lines_above,
+            count,
+            self.width,
+        );
+
+        self.scrollback_buffer_lines =
+            subtract_isize_from_usize(self.scrollback_buffer_lines, transferred_rows_count);
+    }
 }
 
 impl Perform for Grid {
@@ -1600,7 +1530,8 @@ impl Perform for Grid {
                 if params.len() < 3 {
                     return;
                 }
-                self.cursor.pending_styles.link_anchor = self.link_handler.dispatch_osc8(params);
+                self.cursor.pending_styles.link_anchor =
+                    self.link_handler.borrow_mut().dispatch_osc8(params);
             }
 
             // Get/set Foreground, Background, Cursor colors.
@@ -1765,6 +1696,8 @@ impl Perform for Grid {
                     self.clear_all_before_cursor(char_to_replace);
                 } else if clear_type == 2 {
                     self.fill_viewport(char_to_replace);
+                } else if clear_type == 3 {
+                    self.clear_lines_above();
                 }
             };
         } else if c == 'H' || c == 'f' {
@@ -1798,17 +1731,18 @@ impl Perform for Grid {
                         self.bracketed_paste_mode = false;
                     }
                     Some(1049) => {
+                        // leave alternate buffer
                         if let Some((
                             alternative_lines_above,
                             alternative_viewport,
                             alternative_cursor,
-                        )) = &mut self.alternative_lines_above_viewport_and_cursor
+                        )) = &mut self.alternate_lines_above_viewport_and_cursor
                         {
                             std::mem::swap(&mut self.lines_above, alternative_lines_above);
                             std::mem::swap(&mut self.viewport, alternative_viewport);
                             std::mem::swap(&mut self.cursor, alternative_cursor);
                         }
-                        self.alternative_lines_above_viewport_and_cursor = None;
+                        self.alternate_lines_above_viewport_and_cursor = None;
                         self.clear_viewport_before_rendering = true;
                         self.force_change_size(self.height, self.width); // the alternative_viewport might have been of a different size...
                         self.mark_for_rerender();
@@ -1854,6 +1788,7 @@ impl Perform for Grid {
                         self.bracketed_paste_mode = true;
                     }
                     Some(1049) => {
+                        // enter alternate buffer
                         let current_lines_above = std::mem::replace(
                             &mut self.lines_above,
                             VecDeque::with_capacity(*SCROLL_BUFFER_SIZE.get().unwrap()),
@@ -1863,9 +1798,10 @@ impl Perform for Grid {
                             vec![Row::new(self.width).canonical()],
                         );
                         let current_cursor = std::mem::replace(&mut self.cursor, Cursor::new(0, 0));
-                        self.alternative_lines_above_viewport_and_cursor =
+                        self.alternate_lines_above_viewport_and_cursor =
                             Some((current_lines_above, current_viewport, current_cursor));
                         self.clear_viewport_before_rendering = true;
+                        self.scrollback_buffer_lines = self.recalculate_scrollback_buffer_count();
                     }
                     Some(1) => {
                         self.cursor_key_mode = true;
@@ -2139,6 +2075,7 @@ impl Perform for Grid {
 pub struct Row {
     pub columns: VecDeque<TerminalCharacter>,
     pub is_canonical: bool,
+    width: Option<usize>,
 }
 
 impl Debug for Row {
@@ -2155,12 +2092,14 @@ impl Row {
         Row {
             columns: VecDeque::with_capacity(width),
             is_canonical: false,
+            width: None,
         }
     }
     pub fn from_columns(columns: VecDeque<TerminalCharacter>) -> Self {
         Row {
             columns,
             is_canonical: false,
+            width: None,
         }
     }
     pub fn from_rows(mut rows: Vec<Row>, width: usize) -> Self {
@@ -2176,12 +2115,25 @@ impl Row {
     }
     pub fn with_character(mut self, terminal_character: TerminalCharacter) -> Self {
         self.columns.push_back(terminal_character);
+        self.width = None;
         self
     }
     pub fn canonical(mut self) -> Self {
         self.is_canonical = true;
         self
     }
+    pub fn width_cached(&mut self) -> usize {
+        if self.width.is_some() {
+            self.width.unwrap()
+        } else {
+            let mut width = 0;
+            for terminal_character in &self.columns {
+                width += terminal_character.width;
+            }
+            self.width = Some(width);
+            width
+        }
+    }
     pub fn width(&self) -> usize {
         let mut width = 0;
         for terminal_character in &self.columns {
@@ -2221,15 +2173,18 @@ impl Row {
         absolute_index
     }
     pub fn add_character_at(&mut self, terminal_character: TerminalCharacter, x: usize) {
-        match self.width().cmp(&x) {
+        match self.width_cached().cmp(&x) {
             Ordering::Equal => {
                 self.columns.push_back(terminal_character);
+                // this is unwrapped because this always happens after self.width_cached()
+                *self.width.as_mut().unwrap() += terminal_character.width;
             }
             Ordering::Less => {
                 let width_offset = self.excess_width_until(x);
                 self.columns
                     .resize(x.saturating_sub(width_offset), EMPTY_TERMINAL_CHARACTER);
                 self.columns.push_back(terminal_character);
+                self.width = None;
             }
             Ordering::Greater => {
                 // wide-character-aware index, where each character is counted once
@@ -2257,6 +2212,7 @@ impl Row {
                     }
                     _ => {}
                 }
+                self.width = None;
             }
         }
     }
@@ -2273,6 +2229,7 @@ impl Row {
                 self.columns.insert(insert_position, terminal_character);
             }
         }
+        self.width = None;
     }
     pub fn replace_character_at(&mut self, terminal_character: TerminalCharacter, x: usize) {
         // this is much more performant than remove/insert
@@ -2285,12 +2242,15 @@ impl Row {
                 self.columns.insert(x, terminal_character);
             }
         }
+        self.width = None;
     }
     pub fn replace_columns(&mut self, columns: VecDeque<TerminalCharacter>) {
         self.columns = columns;
+        self.width = None;
     }
     pub fn push(&mut self, terminal_character: TerminalCharacter) {
         self.columns.push_back(terminal_character);
+        self.width = None;
     }
     pub fn truncate(&mut self, x: usize) {
         let width_offset = self.excess_width_until(x);
@@ -2298,6 +2258,7 @@ impl Row {
         if truncate_position < self.columns.len() {
             self.columns.truncate(truncate_position);
         }
+        self.width = None;
     }
     pub fn position_accounting_for_widechars(&self, x: usize) -> usize {
         let mut position = x;
@@ -2325,9 +2286,11 @@ impl Row {
         self.columns
             .truncate(from_position_accounting_for_widechars);
         self.columns.append(&mut replace_with);
+        self.width = None;
     }
     pub fn append(&mut self, to_append: &mut VecDeque<TerminalCharacter>) {
         self.columns.append(to_append);
+        self.width = None;
     }
     pub fn drain_until(&mut self, x: usize) -> VecDeque<TerminalCharacter> {
         let mut drained_part: VecDeque<TerminalCharacter> = VecDeque::new();
@@ -2341,6 +2304,7 @@ impl Row {
                 break;
             }
         }
+        self.width = None;
         drained_part
     }
     pub fn replace_and_pad_beginning(&mut self, to: usize, terminal_character: TerminalCharacter) {
@@ -2360,6 +2324,7 @@ impl Row {
             drop(self.columns.drain(0..=to_position_accounting_for_widechars));
         }
         replace_with.append(&mut self.columns);
+        self.width = None;
         self.columns = replace_with;
     }
     pub fn replace_beginning_with(&mut self, mut line_part: VecDeque<TerminalCharacter>) {
@@ -2370,6 +2335,7 @@ impl Row {
             drop(self.columns.drain(0..line_part.len()));
         }
         line_part.append(&mut self.columns);
+        self.width = None;
         self.columns = line_part;
     }
     pub fn len(&self) -> usize {
@@ -2381,6 +2347,7 @@ impl Row {
     pub fn delete_and_return_character(&mut self, x: usize) -> Option<TerminalCharacter> {
         let erase_position = self.absolute_character_index(x);
         if erase_position < self.columns.len() {
+            self.width = None;
             Some(self.columns.remove(erase_position).unwrap()) // TODO: just return the remove part?
         } else {
             None
@@ -2408,6 +2375,7 @@ impl Row {
         if parts.is_empty() {
             parts.push(self.clone());
         }
+        self.width = None;
         parts
     }
 }
