@@ -6,13 +6,17 @@ use std::os::unix::io::RawFd;
 use std::rc::Rc;
 use std::str;
 
+use zellij_utils::input::options::Clipboard;
 use zellij_utils::pane_size::Size;
-use zellij_utils::{input::layout::Layout, position::Position, zellij_tile};
+use zellij_utils::{
+    input::command::TerminalAction, input::layout::Layout, position::Position, zellij_tile,
+};
 
 use crate::{
+    output::Output,
     panes::PaneId,
     pty::{ClientOrTabIndex, PtyInstruction, VteBytes},
-    tab::{Output, Tab},
+    tab::Tab,
     thread_bus::Bus,
     ui::overlay::{Overlay, OverlayWindow, Overlayable},
     wasm_vm::PluginInstruction,
@@ -31,6 +35,8 @@ pub enum ScreenInstruction {
     PtyBytes(RawFd, VteBytes),
     Render,
     NewPane(PaneId, ClientOrTabIndex),
+    TogglePaneEmbedOrFloating(ClientId),
+    ToggleFloatingPanes(ClientId, Option<TerminalAction>),
     HorizontalSplit(PaneId, ClientId),
     VerticalSplit(PaneId, ClientId),
     WriteCharacter(Vec<u8>, ClientId),
@@ -100,6 +106,10 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::PtyBytes(..) => ScreenContext::HandlePtyBytes,
             ScreenInstruction::Render => ScreenContext::Render,
             ScreenInstruction::NewPane(..) => ScreenContext::NewPane,
+            ScreenInstruction::TogglePaneEmbedOrFloating(..) => {
+                ScreenContext::TogglePaneEmbedOrFloating
+            }
+            ScreenInstruction::ToggleFloatingPanes(..) => ScreenContext::ToggleFloatingPanes,
             ScreenInstruction::HorizontalSplit(..) => ScreenContext::HorizontalSplit,
             ScreenInstruction::VerticalSplit(..) => ScreenContext::VerticalSplit,
             ScreenInstruction::WriteCharacter(..) => ScreenContext::WriteCharacter,
@@ -193,10 +203,13 @@ pub(crate) struct Screen {
     colors: Palette,
     draw_pane_frames: bool,
     session_is_mirrored: bool,
+    copy_command: Option<String>,
+    copy_clipboard: Clipboard,
 }
 
 impl Screen {
     /// Creates and returns a new [`Screen`].
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         bus: Bus<ScreenInstruction>,
         client_attributes: &ClientAttributes,
@@ -204,6 +217,8 @@ impl Screen {
         mode_info: ModeInfo,
         draw_pane_frames: bool,
         session_is_mirrored: bool,
+        copy_command: Option<String>,
+        copy_clipboard: Clipboard,
     ) -> Self {
         Screen {
             bus,
@@ -219,6 +234,8 @@ impl Screen {
             default_mode_info: mode_info,
             draw_pane_frames,
             session_is_mirrored,
+            copy_command,
+            copy_clipboard,
         }
     }
 
@@ -356,7 +373,7 @@ impl Screen {
 
     fn close_tab_at_index(&mut self, tab_index: usize) {
         let mut tab_to_close = self.tabs.remove(&tab_index).unwrap();
-        let pane_ids = tab_to_close.get_pane_ids();
+        let pane_ids = tab_to_close.get_all_pane_ids();
         // below we don't check the result of sending the CloseTab instruction to the pty thread
         // because this might be happening when the app is closing, at which point the pty thread
         // has already closed and this would result in an error
@@ -421,9 +438,10 @@ impl Screen {
         for tab_index in tabs_to_close {
             self.close_tab_at_index(tab_index);
         }
+        let serialized_output = output.serialize();
         self.bus
             .senders
-            .send_to_server(ServerInstruction::Render(Some(output)))
+            .send_to_server(ServerInstruction::Render(Some(serialized_output)))
             .unwrap();
     }
 
@@ -491,6 +509,8 @@ impl Screen {
             self.connected_clients.clone(),
             self.session_is_mirrored,
             client_id,
+            self.copy_command.clone(),
+            self.copy_clipboard.clone(),
         );
         tab.apply_layout(layout, new_pids, tab_index, client_id);
         if self.session_is_mirrored {
@@ -585,6 +605,7 @@ impl Screen {
                     panes_to_hide: tab.panes_to_hide.len(),
                     is_fullscreen_active: tab.is_fullscreen_active(),
                     is_sync_panes_active: tab.is_sync_panes_active(),
+                    are_floating_panes_visible: tab.are_floating_panes_visible(),
                     other_focused_clients,
                 });
             }
@@ -607,11 +628,14 @@ impl Screen {
                 active_tab.name = String::new();
             }
             "\u{007F}" | "\u{0008}" => {
-                //delete and backspace keys
+                // delete and backspace keys
                 active_tab.name.pop();
             }
             c => {
-                active_tab.name.push_str(c);
+                // It only allows printable unicode
+                if buf.iter().all(|u| matches!(u, 0x20..=0x7E)) {
+                    active_tab.name.push_str(c);
+                }
             }
         }
         self.update_tabs();
@@ -692,6 +716,8 @@ pub(crate) fn screen_thread_main(
         ),
         draw_pane_frames,
         session_is_mirrored,
+        config_options.copy_command,
+        config_options.copy_clipboard.unwrap_or_default(),
     );
     loop {
         let (event, mut err_ctx) = screen
@@ -733,6 +759,32 @@ pub(crate) fn screen_thread_main(
 
                 screen.render();
             }
+            ScreenInstruction::TogglePaneEmbedOrFloating(client_id) => {
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .toggle_pane_embed_or_floating(client_id);
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+                screen.update_tabs(); // update tabs so that the ui indication will be send to the plugins
+                screen.render();
+            }
+            ScreenInstruction::ToggleFloatingPanes(client_id, default_shell) => {
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .toggle_floating_panes(client_id, default_shell);
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+                screen.update_tabs(); // update tabs so that the ui indication will be send to the plugins
+                screen.render();
+            }
             ScreenInstruction::HorizontalSplit(pid, client_id) => {
                 screen
                     .get_active_tab_mut(client_id)
@@ -1040,7 +1092,7 @@ pub(crate) fn screen_thread_main(
                     }
                     None => {
                         for tab in screen.tabs.values_mut() {
-                            if tab.get_pane_ids().contains(&id) {
+                            if tab.get_all_pane_ids().contains(&id) {
                                 tab.close_pane(id);
                                 break;
                             }
