@@ -1,3 +1,4 @@
+use clap::ArgEnum;
 use serde::{Deserialize, Serialize};
 use std::fmt;
 use std::str::FromStr;
@@ -75,13 +76,14 @@ pub enum Event {
     Key(Key),
     Mouse(Mouse),
     Timer(f64),
-    CopyToClipboard,
+    CopyToClipboard(CopyDestination),
+    SystemClipboardFailure,
     InputReceived,
     Visible(bool),
 }
 
 /// Describes the different input modes, which change the way that keystrokes will be interpreted.
-#[derive(Debug, PartialEq, Eq, Hash, Copy, Clone, EnumIter, Serialize, Deserialize)]
+#[derive(Debug, PartialEq, Eq, Hash, Copy, Clone, EnumIter, Serialize, Deserialize, ArgEnum)]
 pub enum InputMode {
     /// In `Normal` mode, input is always written to the terminal, except for the shortcuts leading
     /// to other modes
@@ -118,6 +120,9 @@ pub enum InputMode {
     /// `Prompt` mode allows interacting with active prompts.
     #[serde(alias = "prompt")]
     Prompt,
+    /// `Tmux` mode allows for basic tmux keybindings functionality
+    #[serde(alias = "tmux")]
+    Tmux,
 }
 
 impl Default for InputMode {
@@ -162,6 +167,7 @@ impl FromStr for InputMode {
             "renametab" => Ok(InputMode::RenameTab),
             "session" => Ok(InputMode::Session),
             "move" => Ok(InputMode::Move),
+            "tmux" => Ok(InputMode::Tmux),
             "prompt" => Ok(InputMode::Prompt),
             "renamepane" => Ok(InputMode::RenamePane),
             e => Err(e.to_string().into()),
@@ -224,6 +230,7 @@ pub struct TabInfo {
     pub panes_to_hide: usize,
     pub is_fullscreen_active: bool,
     pub is_sync_panes_active: bool,
+    pub are_floating_panes_visible: bool,
     pub other_focused_clients: Vec<ClientId>,
 }
 
@@ -265,3 +272,10 @@ impl Default for PluginCapabilities {
         PluginCapabilities { arrow_fonts: true }
     }
 }
+
+#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
+pub enum CopyDestination {
+    Command,
+    Primary,
+    System,
+}
