@@ -11,14 +11,15 @@ pub mod position;
 pub mod setup;
 pub mod shared;
 
+pub use anyhow;
 pub use async_std;
+pub use clap;
 pub use interprocess;
 pub use libc;
 pub use nix;
 pub use serde;
 pub use serde_yaml;
 pub use signal_hook;
-pub use structopt;
 pub use termion;
 pub use vte;
 pub use zellij_tile;
